extends Control


const save_file = "user://data.tres"

signal loadedMode

# Called when the node enters the scene tree for the first time.
func _ready():
	var data = loadStats()

	Singleton.lang = getLang(data.language)
	Singleton.sensitivity = data.sensitivity
	Singleton.doubletap = data.doubletap
	Singleton.showZeros = data.showZeros
	Singleton.countDown = data.countDown
	Singleton.showOutline = data.showOutline
	Singleton.hideSafeBlocks = data.hideSafeBlocks
	Singleton.mode = data.mode

	loadedMode.emit()

	TranslationServer.set_locale(Singleton.lang)

	$Difficulty/Beginner/RichTextLabel.text = "[center]" + tr("EASY") + " 5x5x5" + "[/center]"
	$Difficulty/Intermediate/RichTextLabel.text = "[center]" + tr("MEDIUM") + " 9x9x9" + "[/center]"
	$Difficulty/Master/RichTextLabel.text = "[center]" + tr("HARD") + " 15x15x15" + "[/center]"
	$Stats/Area2D/RichTextLabel.text = "[center]" + tr("STATS") + "[/center]"
	$Settings/ConfigCircle/RichTextLabel.text = "[center]" + tr("SETTINGS") + "[/center]"
	$Tutorial/TutorialCircle/RichTextLabel.text = "[center]" + tr("TUTORIAL") + "[/center]"


func getLang(langNum):
	match langNum:
		0:
			return "en"
		1:
			return "de"
		2:
			return "esp"


func loadStats():
	if ResourceLoader.exists(save_file):
		return updateStatsFile(load(save_file))
	return GameData.new()


func updateStatsFile(data):
	if "version" in data and data.version == Singleton.dataVersion:
		return data
	else:
		var newData = GameData.new()
		newData.playedGames = data.playedGames
		newData.wonGames = data.wonGames
		newData.totalMines = data.totalMines

		newData.fastestEasy = data.fastestEasy
		newData.fastestEasyMines = data.fastestEasyMines
		newData.wonEasy = data.wonEasy
		newData.totalEasy = data.totalEasy

		newData.fastestMediumMines = data.fastestMediumMines
		newData.fastestMedium = data.fastestMedium
		newData.wonMedium = data.wonMedium
		newData.totalMedium = data.totalMedium

		newData.fastestHardMines = data.fastestHardMines
		newData.fastestHard = data.fastestHard
		newData.wonHard = data.wonHard
		newData.totalHard = data.totalHard

		newData.sound = data.sound
		newData.vibrate = data.vibrate

		if "sensitivity" in data:
			newData.sensitivity = data.sensitivity
			newData.doubletap = data.doubletap
			newData.showZeros = data.showZeros

		if "showOutline" in data:
			newData.showOutline = data.showOutline
			newData.hideSafeBlocks = data.hideSafeBlocks

		if "language" in data:
			newData.language = data.language
		
		if "mode" in data:
			newData.mode = data.mode

		saveStats(newData)
		return newData


func saveStats(data):
	ResourceSaver.save(data, save_file)


func _on_save() -> void:
	var data = loadStats()
	data.mode = Singleton.mode
	print(Singleton.mode)
	saveStats(data)
