extends Node3D

var white = preload("res://assets/white.tres")
var orange = preload("res://assets/orange.tres")
var red = preload("res://assets/red.tres")

signal exploded
signal cleared
signal defused
signal refused
signal passMine

var mine = false
var hovering = false
var triggered = false
var marked = false
var dead = false
var disappearing = false
var beenBig = false
var beenFinalCleared = false
var mod = 1
var proxy = false
var active = true

var puristMode = !Singleton.mode

var neighbors = []
var mineCount = 0
var initialMineCount = 0

var dragging = false


func _ready():
	get_mine_count()
	if !mine:
		$MineCount.text = str(mineCount)
	else:
		$MineCount.text = ""

	initialMineCount = mineCount


func _input(event):
	if active and Singleton.doubletap and !dead and !triggered and !Singleton.skip:
		if event is InputEventScreenTouch:
			if event.pressed:
				Singleton.dragging = false
			if event.double_tap:
				if hovering and !Singleton.dragging:
					if !marked:
						if Singleton.first_press:
							Singleton.first_press = false
							if mine:
								pass_mine()
						if mine:
							$BoxArea/BoxMesh.material_override = red
							if Singleton.vibrate:
								Input.vibrate_handheld(1000)
							final_clear()
							dead = true
							exploded.emit()
						else:
							dissolve(true)
					else:
						marked = false
						$BoxArea/BoxMesh.material_override = white
						refused.emit()
						for neighbor in neighbors:
							if !neighbor.mine:
								neighbor.mineCount += 1
								neighbor.update()


func _process(delta):
	if active:
		if disappearing and $BoxArea/BoxMesh.scale.y > 0:
			if marked:
				$BoxArea/BoxMesh.scale += Vector3(0, 1 * delta * mod, 0)
			else:
				$BoxArea/BoxMesh.scale += Vector3(4 * delta * mod, 4 * delta * mod, 4 * delta * mod)
			if $BoxArea/BoxMesh.scale.y > 1.2 and !beenBig:
				beenBig = true
				mod *= -1
			return
		elif disappearing:
			$BoxArea.hide()
			if marked:
				for neighbor in neighbors:
					if !neighbor.mine:
						neighbor.initialMineCount -= 1
						neighbor.update(false)

				active = false
			elif initialMineCount == 0:
				active = false

			disappearing = false

		if !triggered and !Singleton.skip:
			if Input.is_action_just_pressed("left_mouse"):
				Singleton.dragging = false

			if !Singleton.dragging:
				if !Singleton.doubletap && Input.is_action_just_released("left_mouse"):
					if hovering and !Input.is_action_just_pressed("right_mouse"):
						if !marked:
							if Singleton.first_press:
								Singleton.first_press = false
								if mine:
									pass_mine()

							if mine:
								$BoxArea/BoxMesh.material_override = red
								if Singleton.vibrate:
									Input.vibrate_handheld(1000)
								final_clear()
								dead = true
								exploded.emit()
							else:
								# For the *real* Minesweeper experience,
								# this would have to be "false"
								dissolve(false if puristMode else true)
						else:
							marked = false
							$BoxArea/BoxMesh.material_override = white
							refused.emit()
							for neighbor in neighbors:
								if !neighbor.mine:
									neighbor.mineCount += 1
									neighbor.update()

				if Input.is_action_just_pressed("right_mouse"):
					if hovering:
						if marked:
							marked = false
							$BoxArea/BoxMesh.material_override = white
							refused.emit()
							for neighbor in neighbors:
								if !neighbor.mine:
									neighbor.mineCount += 1
									neighbor.update()

						else:
							marked = true
							defused.emit()
							$BoxArea/BoxMesh.material_override = orange
							for neighbor in neighbors:
								if !neighbor.mine:
									neighbor.mineCount -= 1
									neighbor.update()
							check_islands()


func neighboringMines():
	var badNeighbors = []
	for neighbor in neighbors:
		if neighbor.mine:
			badNeighbors.append(neighbor)
	
	return badNeighbors


func is_safe():
	for neighbor in neighbors:
		if !((neighbor.triggered && neighbor.mineCount == 0) || neighbor.marked):
			return false

	return true


func check_islands():
	if Singleton.hideSafeBlocks:
		var checkedNeighbors = []
		var neighborsToCheck = [self]
		if !is_safe():
			return

		while !neighborsToCheck.is_empty():
			var newNeighbors = []
			for neighbor in neighborsToCheck:
				var potentialBaddies = neighbor.neighboringMines()
				for mine in potentialBaddies:
					if mine not in checkedNeighbors:
						newNeighbors.append(mine)

				checkedNeighbors.append(neighbor)

			neighborsToCheck = newNeighbors

		for cube in checkedNeighbors:
			if !cube.is_safe():
				return

		for cube in checkedNeighbors:
			cube.disappearing = true
			await get_tree().create_timer(0.01).timeout


func _on_box_area_mouse_entered():
	if !dead:
		hovering = true


func _on_box_area_mouse_exited():
	if !dead:
		hovering = false


func get_mine_count():
	for neighbor in neighbors:
		if neighbor.mine == true:
			mineCount += 1


func pressed():
	if marked:
		marked = false
		refused.emit()
		for neighbor in neighbors:
			if !neighbor.mine:
				neighbor.mineCount += 1
				neighbor.update()

	else:
		if !triggered:
			if Singleton.sound:
				$Plopper.play()
			cleared.emit()
		if mineCount != 0:
			$MineCount.show()
		triggered = true


func dissolve(kill: bool):
	if (!triggered and !marked and mineCount == 0 and !mine) or (!triggered and kill):
		pressed()

		for neighbor in neighbors:
			if !neighbor.triggered and !neighbor.marked and !neighbor.mine:
				await get_tree().create_timer(0.02).timeout
				neighbor.dissolve(false)
			elif neighbor.marked && neighbor.mine:
				check_islands()

	elif !mine and !marked:
		pressed()

	if !mine:
		disappearing = true

	update()


func update(check = true):
	var mc = mineCount if Singleton.countDown || (mineCount == 0 && Singleton.countDown) else initialMineCount
	$MineCount.text = str(mc)

	if mineCount == 0:
		$MineCount.set_modulate(Color.LIGHT_GRAY)
	elif mc > 4:
		$MineCount.set_modulate(Color.CRIMSON)
	elif mc > 2:
		$MineCount.set_modulate(Color.GOLD)
	elif mc > 0:
		$MineCount.set_modulate(Color.BLACK)

	if (mineCount == 0 && !Singleton.showZeros) || initialMineCount == 0:
		$MineCount.hide()
	elif triggered:
		$MineCount.show()

	if check:
		for neighbor in neighbors:
			if neighbor.marked && neighbor.mine:
				neighbor.check_islands()


func pass_mine():
	mine = false
	update()
	for neighbor in neighbors:
		neighbor.mineCount -= 1
		neighbor.initialMineCount -= 1
		neighbor.update()
	passMine.emit()


func recieve_mine():
	mine = true
	$MineCount.text = ""
	for neighbor in neighbors:
		neighbor.mineCount += 1
		neighbor.initialMineCount += 1
		neighbor.update()


func final_clear():
	if triggered and mineCount != 0:
		$MineCount.hide()
	if !mine:
		triggered = true
		disappearing = true
	else:
		$BoxArea/BoxMesh.material_override = red

	beenFinalCleared = true

	neighbors.shuffle()
	for neighbor in neighbors:
		if !neighbor.beenFinalCleared:
			neighbor.final_clear()
		await get_tree().create_timer(0.1).timeout


func _on_box_area_area_entered(_area):
	if !triggered:
		hide()


func _on_box_area_area_exited(_area):
	show()
