extends VBoxContainer

const save_file = "user://data.tres"

var sound = true
var vibrate = true
var doubletap = false
var showZeros = false
var countDown = true
var showOutline = true
var hideSafeBlocks = false

var soundPos = 0
var soundWidth = 150
var vibratePos = 0
var vibrateWidth = 150
var doubletapPos = 0
var doubletapWidth = 150
var showZerosPos = 0
var showZerosWidth = 150
var countDownPos = 0
var countDownWidth = 150
var showOutlinePos = 0
var showOutlineWidth = 150
var hideSafeBlocksPos = 0
var hideSafeBlocksWidth = 150
var sensitivity = 10


func _ready():
	var data = loadStats()
	sound = data.sound
	vibrate = data.vibrate
	doubletap = data.doubletap
	showZeros = data.showZeros
	countDown = data.countDown
	showOutline = data.showOutline
	hideSafeBlocks = data.hideSafeBlocks

	$SliderPos/HSlider.value = Singleton.sensitivity
	$SliderPos/SliderText.position.y -= $SliderPos/SliderText.get_content_height()
	Singleton.lang = getLang(data.language)
	call_deferred("setPositions")
	updateText()


func setPositions() -> void:
	soundPos = $SoundPos/Sounds.global_position.y
	vibratePos = $VibratePos/Vibrate.global_position.y
	doubletapPos = $DoubletapPos/Doubletap.global_position.y
	showZerosPos = $ShowZerosPos/ShowZeros.global_position.y
	countDownPos = $CountDownPos/CountDown.global_position.y
	showOutlinePos = $ShowOutlinePos/ShowOutline.global_position.y
	hideSafeBlocksPos = $HideSafeBlocksPos/HideSafeBlocks.global_position.y

	setWidth()
	queue_redraw()


func setWidth() -> void:
	soundWidth = $SoundPos/Sounds/RichTextLabel.get_content_width()
	vibrateWidth = $VibratePos/Vibrate/RichTextLabel.get_content_width()
	doubletapWidth = $DoubletapPos/Doubletap/RichTextLabel.get_content_width()
	showZerosWidth = $ShowZerosPos/ShowZeros/RichTextLabel.get_content_width()
	countDownWidth = $CountDownPos/CountDown/RichTextLabel.get_content_width()
	showOutlineWidth = $ShowOutlinePos/ShowOutline/RichTextLabel.get_content_width()
	hideSafeBlocksWidth = $HideSafeBlocksPos/HideSafeBlocks/RichTextLabel.get_content_width()


func settingByIdx(idx: int) -> bool:
	return [sound, vibrate, doubletap, showZeros, countDown, showOutline, hideSafeBlocks][idx]


func posByIdx(idx: int) -> float:
	return [soundPos, vibratePos, doubletapPos, showZerosPos, countDownPos, showOutlinePos, hideSafeBlocksPos][idx]


func widthByIdx(idx: int) -> float:
	return [soundWidth, vibrateWidth, doubletapWidth, showZerosWidth, countDownWidth, showOutlineWidth, hideSafeBlocksWidth][idx]


func _draw():
	for idx in range(7):
		var pos = posByIdx(idx)
		var width = widthByIdx(idx)
		var screenWidth = get_viewport().size.x
		var from = -(width / 2) - 20 + screenWidth / 2
		var to = width / 2 + 20 + screenWidth / 2
		draw_line(Vector2(from, pos), Vector2(to, pos), Color.BLACK, 140)
		draw_circle(Vector2(from, pos), 70, Color.BLACK)
		draw_circle(Vector2(to, pos), 70, Color.BLACK)

		if settingByIdx(idx):
			draw_line(Vector2(from, pos), Vector2(to, pos), Color.WHITE, 120)
			draw_circle(Vector2(from, pos), 60, Color.WHITE)
			draw_circle(Vector2(to, pos), 60, Color.WHITE)
		else:
			draw_line(Vector2(from, pos), Vector2(to, pos), Color.DARK_GRAY, 120)
			draw_circle(Vector2(from, pos), 60, Color.DARK_GRAY)
			draw_circle(Vector2(to, pos), 60, Color.DARK_GRAY)


func _on_config_toggle_sound():
	sound = !sound
	saveStats()
	queue_redraw()


func _on_config_toggle_vibrate():
	vibrate = !vibrate
	saveStats()
	queue_redraw()


func saveStats():
	var data = loadStats()
	data.sound = sound
	data.vibrate = vibrate
	data.language = enumerateLang(Singleton.lang)
	data.sensitivity = sensitivity
	data.doubletap = doubletap
	data.showZeros = showZeros
	data.countDown = countDown
	data.showOutline = showOutline
	data.hideSafeBlocks = hideSafeBlocks

	ResourceSaver.save(data, save_file)


func enumerateLang(lang):
	match lang:
		"en":
			return 0
		"de":
			return 1
		"esp":
			return 2


# For when I add new things to the gamedata
func updateStatsFile(data):
	if "version" in data and data.version == Singleton.dataVersion:
		return data
	else:
		var newData = GameData.new()
		newData.playedGames = data.playedGames
		newData.wonGames = data.wonGames
		newData.totalMines = data.totalMines

		newData.fastestEasy = data.fastestEasy
		newData.fastestEasyMines = data.fastestEasyMines
		newData.wonEasy = data.wonEasy
		newData.totalEasy = data.totalEasy

		newData.fastestMediumMines = data.fastestMediumMines
		newData.fastestMedium = data.fastestMedium
		newData.wonMedium = data.wonMedium
		newData.totalMedium = data.totalMedium

		newData.fastestHardMines = data.fastestHardMines
		newData.fastestHard = data.fastestHard
		newData.wonHard = data.wonHard
		newData.totalHard = data.totalHard

		newData.sound = data.sound
		newData.vibrate = data.vibrate

		if "sensitivity" in data:
			newData.sensitivity = data.sensitivity
			newData.doubletap = data.doubletap
			newData.showZeros = data.showZeros
			newData.countDown = data.countDown

		if "showOutline" in data:
			newData.showOutline = data.showOutline
			newData.hideSafeBlocks = data.hideSafeBlocks

		if "language" in data:
			newData.language = data.language

		ResourceSaver.save(data, save_file)
		return newData


func loadStats():
	if ResourceLoader.exists(save_file):
		return updateStatsFile(load(save_file))
	return GameData.new()


func updateText():
	$SoundPos/Sounds/RichTextLabel.text 	   =  tr("SOUND")
	$VibratePos/Vibrate/RichTextLabel.text 	   =  tr("VIBRATION")
	$DoubletapPos/Doubletap/RichTextLabel.text =  tr("DOUBLETAP")
	$ShowZerosPos/ShowZeros/RichTextLabel.text =  tr("SHOWZEROS")
	$CountDownPos/CountDown/RichTextLabel.text =  tr("COUNTDOWN")
	$ShowOutlinePos/ShowOutline/RichTextLabel.text = tr("SHOWOUTLINE")
	$HideSafeBlocksPos/HideSafeBlocks/RichTextLabel.text = tr("HIDESAFEBLOCKS")
	$SliderPos/SliderText.text = tr("WIGGLEROOM")

	setWidth()


func _on_config_reset_text():
	updateText()
	saveStats()
	queue_redraw()


func getLang(langNum):
	match langNum:
		0:
			return "en"
		1:
			return "de"
		2:
			return "esp"


func _on_h_slider_drag_ended(value_changed):
	if value_changed:
		sensitivity = $SliderPos/HSlider.value
		saveStats()


func _on_config_toggle_count_down() -> void:
	countDown = !countDown
	saveStats()
	queue_redraw()


func _on_config_toggle_doubletap() -> void:
	doubletap = !doubletap
	saveStats()
	queue_redraw()


func _on_config_toggle_show_zeros() -> void:
	showZeros = !showZeros
	saveStats()
	queue_redraw()


func _on_config_toggle_show_outline() -> void:
	showOutline = !showOutline
	saveStats()
	queue_redraw()


func _on_config_toggle_hide_safe_blocks() -> void:
	hideSafeBlocks = !hideSafeBlocks
	saveStats()
	queue_redraw()
