extends Node3D

const save_file = "user://data.tres"

var Box   = preload("res://scenes/Box.tscn")
var Ghost = preload("res://scenes/Ghost.tscn")

var size  = 25
var shape = "cube"
var showOutline  = true
var activeBlocks = 0
var activeMines  = 0
var totalMines   = 0
var nonMines     = []
var gameTime     = 0.0

signal exploded
signal mineCount(count)
signal won(time)


func _process(delta):
	if !Singleton.first_press:
		gameTime += delta


func _ready():
	var settings = loadStats()
	Singleton.sound = settings.sound
	Singleton.vibrate = settings.vibrate

	var field = []

	if showOutline:
		var ghost = Ghost.instantiate()
		ghost.scale = Vector3i(size, size, size)
		add_child(ghost)

	position = Vector3i(float(size) / 2, float(size) / 2, float(size) / 2)

	var hs = int(size / 2)
	var mod = 0

	if hs * 2 < size:
		mod = 1

	var dummy = Box.instantiate()
	dummy.triggered = true
	var dummyRow = []
	var dummySlice = []

	for x in range(0, size + 2):
		dummyRow.append(dummy)
	for x in range(0, size + 2):
		dummySlice.append(dummyRow)

	if shape == "cube":
		activeBlocks = size ** 3
		field.append(dummySlice)
		for x in range(-hs, hs + mod):
			var slice = []
			slice.append(dummyRow)
			for y in range(-hs, hs + mod):
				var row = []
				row.append(dummy)
				for z in range(-hs, hs + mod):

					var box = Box.instantiate()
					box.exploded.connect(die)
					box.cleared.connect(clearBox)
					box.defused.connect(markMine)
					box.refused.connect(rearmMine)
					box.passMine.connect(passMine)
					var rand = randi_range(1, 10)

					box.position = Vector3i(x, y, z)

					if rand == 5:
						box.mine = true
						activeMines += 1
					else:
						nonMines.append(box)

					row.append(box)
					

				row.append(dummy)
				slice.append(row)
			slice.append(dummyRow)

			field.append(slice)
		field.append(dummySlice)
	totalMines = activeMines

	for z in range(0, size + 2):
		for y in range(0, size + 2):
			for x in range(0, size + 2):
				if x == 0 or x == size + 1 or y == 0 or y == size + 1 or z == 0 or z == size + 1:
					pass
				else:
					field[z][y][x].neighbors = [
						field[z][y][x + 1],
						field[z][y][x - 1],

						field[z][y + 1][x + 1],
						field[z][y + 1][x - 1],
						field[z][y + 1][x],
						field[z][y - 1][x + 1],
						field[z][y - 1][x],
						field[z][y - 1][x - 1], 

						field[z + 1][y][x + 1],
						field[z + 1][y][x],
						field[z + 1][y][x - 1],
						field[z + 1][y + 1][x + 1],
						field[z + 1][y + 1][x - 1],
						field[z + 1][y + 1][x],
						field[z + 1][y - 1][x + 1],
						field[z + 1][y - 1][x],
						field[z + 1][y - 1][x - 1], 

						field[z - 1][y][x + 1],
						field[z - 1][y][x],
						field[z - 1][y][x - 1],
						field[z - 1][y + 1][x + 1],
						field[z - 1][y + 1][x - 1],
						field[z - 1][y + 1][x],
						field[z - 1][y - 1][x + 1],
						field[z - 1][y - 1][x],
						field[z - 1][y - 1][x - 1]
					].filter(
						func(realBoxes): return !realBoxes.triggered
					)

					add_child(field[z][y][x])

	updateMines()
	checkState()


func die():
	exploded.emit()
	var data = loadStats()
	data.totalMines += totalMines - activeMines
	data.playedGames += 1
	if size == 5:
		data.totalEasy += 1
	elif size == 9:
		data.totalMedium += 1
	else:
		data.totalHard += 1

	saveStats(data)


func clearBox():
	activeBlocks -= 1
	checkState()


func markMine():
	activeMines -= 1
	checkState()
	updateMines()


func rearmMine():
	activeMines += 1
	updateMines()


func updateMines():
	mineCount.emit(activeMines)


func checkState():
	if activeBlocks == totalMines and activeMines == 0:
		var totalTime = round(gameTime * pow(10.0, 2)) / pow(10.0, 2)
		won.emit(totalTime)
		var data = loadStats()
		data.totalMines += totalMines
		data.playedGames += 1
		data.wonGames += 1
		if size == 5:
			data.totalEasy += 1
			data.wonEasy += 1
			if data.fastestEasyMines == -1 or data.fastestEasy > totalTime:
				data.fastestEasy = totalTime
				data.fastestEasyMines = totalMines
		elif size == 9:
			data.totalMedium += 1
			data.wonMedium += 1
			if data.fastestMediumMines == -1 or data.fastestMedium > totalTime:
				data.fastestMedium = totalTime
				data.fastestMediumMines = totalMines
		else:
			data.totalHard += 1
			data.wonHard += 1
			if data.fastestHardMines == -1 or data.fastestHard > totalTime:
				data.fastestHard = totalTime
				data.fastestHardMines = totalMines

		saveStats(data)


func passMine():
	nonMines.pick_random().recieve_mine()


func saveStats(data):
	ResourceSaver.save(data, save_file)


# For when I add new things to the gamedata
func updateStatsFile(data):
	if "version" in data and data.version == Singleton.dataVersion:
		return data
	else:
		var newData = GameData.new()
		newData.playedGames = data.playedGames
		newData.wonGames = data.wonGames
		newData.totalMines = data.totalMines
		
		newData.fastestEasy = data.fastestEasy
		newData.fastestEasyMines = data.fastestEasyMines
		newData.wonEasy = data.wonEasy
		newData.totalEasy = data.totalEasy

		newData.fastestMediumMines = data.fastestMediumMines
		newData.fastestMedium = data.fastestMedium
		newData.wonMedium = data.wonMedium
		newData.totalMedium = data.totalMedium
		
		newData.fastestHardMines = data.fastestHardMines
		newData.fastestHard = data.fastestHard
		newData.wonHard = data.wonHard
		newData.totalHard = data.totalHard
		
		newData.sound = data.sound
		newData.vibrate = data.vibrate
		
		if "language" in data:
			newData.language = data.language

		saveStats(newData)
		return newData


func loadStats():
	if ResourceLoader.exists(save_file):
		return updateStatsFile(load(save_file))
	return GameData.new()
