extends WorldEnvironment

var MineField = preload("res://scenes/MineField.tscn")
var center: int
var centerPoint: Vector3
var upVec = Vector3(0, 1, 0)
var zooming = false
var over = false
var reallyOver = false
var pressPoint = Vector2.ZERO

func _ready():
	get_tree().quit_on_go_back = false
	genesis()

	center = Singleton.fieldSize / 2
	centerPoint = Vector3(center, center, center)

	$PlayerSphere.position = centerPoint
	$PlayerSphere/Cam.look_at(centerPoint, upVec)
	Singleton.first_press = true


func genesis():

	var minefield = MineField.instantiate()

	if Singleton.fieldSize == 5:
		$PlayerSphere/Cam.position = Vector3(1.5, 5, 10)
	elif Singleton.fieldSize == 9:
		$PlayerSphere/Cam.position = Vector3(2.5, 7.5, 14)
	else:
		$PlayerSphere/Cam.position = Vector3(4, 10, 18)

	minefield.size = Singleton.fieldSize
	minefield.shape = "cube"
	minefield.showOutline = Singleton.showOutline

	minefield.exploded.connect(dead)
	minefield.mineCount.connect(updateCount)
	minefield.won.connect(win)
	add_child(minefield)


func _notification(what):
	match what:
		NOTIFICATION_WM_GO_BACK_REQUEST:
			get_tree().change_scene_to_file("res://scenes/Settings.tscn")


func _input(event):
	var delta = 0.005

	var is_drag = event is InputEventScreenDrag or event is InputEventMouseMotion
	var is_press = !is_drag and event.double_tap if Singleton.doubletap else event.is_pressed()

	if event is InputEventMagnifyGesture:

		Singleton.dragging = true
		zooming = true

		if reallyOver:
			reallyOver = false

		$PlayerSphere.scale *= (event.factor - 2) * -1
		$PlayerSphere.scale = clamp(
			$PlayerSphere.scale, 
			Vector3(.1, .1, .1), 
			Vector3(5, 5, 5)
		)


	elif event is InputEventPanGesture:
		Singleton.dragging = true
		zooming = true
		$PlayerSphere.rotate_object_local(Vector3.FORWARD, event.delta.x * delta)


	if event is InputEventScreenDrag and !zooming:
		if pressPoint.distance_to(event.position) > Singleton.sensitivity:
			Singleton.dragging = true

		if reallyOver:
			reallyOver = false

		$PlayerSphere.rotate_object_local(Vector3.LEFT, event.relative.y * delta)
		$PlayerSphere.rotate_object_local(Vector3.DOWN, event.relative.x * delta)

	if event is InputEventScreenTouch:
		if is_press:
			pressPoint = event.position
			zooming = false
			if over:
				reallyOver = true

		else:
			if reallyOver:
				get_tree().change_scene_to_file("res://scenes/LoadingScreenLight.tscn")



func dead():
	$PlayerSphere/Cam/Menu/UI/Text.text =  "[center]" + tr("EXPLODED") + "[/center]"
	$PlayerSphere/Cam/Menu/UI.show()
	over = true


func updateCount(count):
	$PlayerSphere/Cam/UI/UILayer/MinesCount.text = "[center] " + str(count) + " "


func win(time):
	$PlayerSphere/Cam/Menu/UI/Text.text = "[center]" + tr("WON_1") + str(time) + tr("WON_2") + "[/center]"
	$PlayerSphere/Cam/Menu/UI/Text.add_theme_color_override("default_color", Color("green"))
	$PlayerSphere/Cam/Menu/UI.show()
	over = true
